source("read_mar.r")
# マルコフ基底 
# 4ti2 のコマンド markov の出力結果とする
markov_b <- read_mar("covariate_mat1.mar")

# markov_b を -1 倍したものを付け加える
tm <- t(markov_b)
tmm <- -tm
markov_b <- matrix(append(tm, tmm), nrow(tm))
markov_b <- t(markov_b)

# p(y + z)/p(y) の計算 (z はマルコフ基底のいずれか)
# p(y) = 1/Z 1/(y[1]! ... y[length(y)]!)
prob_r <- function(y, z) {
	len <- length(y)
	r <- 1
	for (i in 1:len) {
		# マルコフ基底の成分が正, 負の場合で場合分けして計算
		if (z[i] > 0) {
			for (t in 1:z[i])
				r <- r / (y[i] + t)
		} else if (z[i] < 0) {
			for (t in 0:(z[i]+1))
				r <- r * (y[i] + t)
		}
	}
	return(r)
}

# 尤度比検定統計量の計算
my_likelihood_ratio <- function(x, fitted_v) {
	n <- length(x)
	sum <- 0
	for (i in 1:n) {
		sum <- sum + x[i] * log(x[i]/fitted_v[i])
	}
	sum <- 2 * sum
	return(sum)
}

test_my_likelihood_ratio <- function()
{
x <- c(69, 31, 55, 149, 46, 43, 118, 30, 43, 45, 71, 380, 37, 36, 212, 52)
fv <- c(64.52677, 47.25345, 53.14603, 151.07960, 30.42595, 46.79383, 115.24100, 32.53337,
        49.42430, 46.13193, 70.90290, 360.53502, 35.18867, 30.25510, 232.14438, 51.41770)
	return(my_likelihood_ratio(x, fv))
}

# 実験データの読み出し
dat <- read.table("2_7-3.dat", sep=",", header=T)
dat.glm <- glm(x~A+B+C+D+E+F+G+A*C+B*D, dat, family="poisson")
fv <- fitted(dat.glm)
xx <- as.vector(t(dat["x"]))    

# mcmc 
# 引数
# n      : 取り出す数
# burnin : 最初からいくら捨てるか
# state  : 最初の状態 
cov1_mcmc <- function(n = 100000, burnin = 50000) {
	state <- xx
	lratio <- my_likelihood_ratio(state, fv)
	cat("likelihood ratio stat :\n")
	print(lratio)

	n_mar <- nrow(markov_b)
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:burnin) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
	}

	count <- 0
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:n) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
		tlratio <- my_likelihood_ratio(state, fv)
		if (tlratio > lratio) {
			count <- count + 1
			print("*") # 条件に合うサンプル 
		}
		print(state) # サンプルの表示
	}
	cat("n\n")
	print(n)
	cat("count\n")
	print(count)
	cat("p値の近似値\n")
	print(count/n)
}

cat("cov1_mcmc(n, burnin) で実行\n")
cat("e.g. cov1_mcmc(10000, 10000)\n")


# 漸近的 p-value 0.00405
#> summary(dat.glm)
#
#Call:
#glm(formula = x ~ A + B + C + D + E + F + G + A * C + B * D,
#    family = "poisson", data = dat)
#
#Deviance Residuals:
#     Min        1Q    Median        3Q       Max
#-2.52419  -0.47818   0.04629   0.36474   2.62292
#
#Coefficients:
#            Estimate Std. Error z value Pr(>|z|)
#(Intercept)  4.17140    0.03393 122.946  < 2e-16 ***
#A           -0.10842    0.03250  -3.337 0.000848 ***
#B            0.19039    0.03015   6.315 2.70e-10 ***
#C           -0.42315    0.03352 -12.625  < 2e-16 ***
#D            0.01264    0.03086   0.409 0.682193
#E            0.10302    0.03206   3.213 0.001312 **
#F           -0.04236    0.02784  -1.522 0.128070
#G            0.38144    0.03352  11.380  < 2e-16 ***
#A:C          0.18109    0.03318   5.457 4.83e-08 ***
#B:D         -0.29896    0.03380  -8.845  < 2e-16 ***
#---
#Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#
#(Dispersion parameter for poisson family taken to be 1)
#
#    Null deviance: 1021.255  on 15  degrees of freedom
#Residual deviance:   19.093  on  6  degrees of freedom
#AIC: 135.26
#
#Number of Fisher Scoring iterations: 4
#
#> 1-pchisq(19.063, df=6)
#[1] 0.004058548
#>                                                                  
